@@ -1,19 +1,48 @@
 # -*- test-case-name: twisted.names.test.test_dns -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 DNS protocol implementation.
 
 Future Plans:
     - Get rid of some toplevels, maybe.
-    - Put in a better lookupRecordType implementation.
 
 @author: Moshe Zadka
-@author: Jp Calderone
+@author: Jean-Paul Calderone
 """
 
+__all__ = [
+    'IEncodable', 'IRecord',
+
+    'A', 'A6', 'AAAA', 'AFSDB', 'CNAME', 'DNAME', 'HINFO',
+    'MAILA', 'MAILB', 'MB', 'MD', 'MF', 'MG', 'MINFO', 'MR', 'MX',
+    'NAPTR', 'NS', 'NULL', 'PTR', 'RP', 'SOA', 'SPF', 'SRV', 'TXT', 'WKS',
+
+    'ANY', 'CH', 'CS', 'HS', 'IN',
+
+    'ALL_RECORDS', 'AXFR', 'IXFR',
+
+    'EFORMAT', 'ENAME', 'ENOTIMP', 'EREFUSED', 'ESERVER',
+
+    'Record_A', 'Record_A6', 'Record_AAAA', 'Record_AFSDB', 'Record_CNAME',
+    'Record_DNAME', 'Record_HINFO', 'Record_MB', 'Record_MD', 'Record_MF',
+    'Record_MG', 'Record_MINFO', 'Record_MR', 'Record_MX', 'Record_NAPTR',
+    'Record_NS', 'Record_NULL', 'Record_PTR', 'Record_RP', 'Record_SOA',
+    'Record_SPF', 'Record_SRV', 'Record_TXT', 'Record_WKS',
+
+    'QUERY_CLASSES', 'QUERY_TYPES', 'REV_CLASSES', 'REV_TYPES', 'EXT_QUERIES',
+
+    'Charstr', 'Message', 'Name', 'Query', 'RRHeader', 'SimpleRecord',
+    'DNSDatagramProtocol', 'DNSMixin', 'DNSProtocol',
+
+    'OK', 'OP_INVERSE', 'OP_NOTIFY', 'OP_QUERY', 'OP_STATUS', 'OP_UPDATE',
+    'PORT',
+
+    'AuthoritativeDomainError', 'DNSQueryTimeoutError', 'DomainError',
+    ]
+
+
 # System imports
 import warnings
 
@@ -53,6 +82,7 @@ SRV = 33
 NAPTR = 35
 A6 = 38
 DNAME = 39
+SPF = 99
 
 QUERY_TYPES = {
     A: 'A',
@@ -80,7 +110,8 @@ QUERY_TYPES = {
     SRV: 'SRV',
     NAPTR: 'NAPTR',
     A6: 'A6',
-    DNAME: 'DNAME'
+    DNAME: 'DNAME',
+    SPF: 'SPF'
 }
 
 IXFR, AXFR, MAILB, MAILA, ALL_RECORDS = range(251, 256)
@@ -1391,6 +1422,9 @@ class Record_TXT(tputil.FancyEqMixin, tputil.FancyStrMixin):
 
     @type data: C{list} of C{str}
     @ivar data: Freeform text which makes up this record.
+    
+    @type ttl: C{int}
+    @ivar ttl: The maximum number of seconds which this record should be cached.
     """
     implements(IEncodable, IRecord)
 
@@ -1419,8 +1453,8 @@ class Record_TXT(tputil.FancyEqMixin, tputil.FancyStrMixin):
             soFar += L + 1
         if soFar != length:
             log.msg(
-                "Decoded %d bytes in TXT record, but rdlength is %d" % (
-                    soFar, length
+                "Decoded %d bytes in %s record, but rdlength is %d" % (
+                    soFar, self.fancybasename, length
                 )
             )
 
@@ -1430,7 +1464,27 @@ class Record_TXT(tputil.FancyEqMixin, tputil.FancyStrMixin):
 
 
 
+class Record_SPF(Record_TXT):
+    """
+    Structurally, freeform text. Semantically, a policy definition, formatted
+    as defined in U{rfc 4408<http://www.faqs.org/rfcs/rfc4408.html>}.
+    
+    @type data: C{list} of C{str}
+    @ivar data: Freeform text which makes up this record.
+    
+    @type ttl: C{int}
+    @ivar ttl: The maximum number of seconds which this record should be cached.
+    """
+    TYPE = SPF
+    fancybasename = 'SPF'
+
+
+
 class Message:
+    """
+    L{Message} contains all the information represented by a single
+    DNS request or response.
+    """
     headerFmt = "!H2B4H"
     headerSize = struct.calcsize(headerFmt)
 
@@ -1545,8 +1599,32 @@ class Message:
             list.append(header)
 
 
+    # Create a mapping from record types to their corresponding Record_*
+    # classes.  This relies on the global state which has been created so
+    # far in initializing this module (so don't define Record classes after
+    # this).
+    _recordTypes = {}
+    for name in globals():
+        if name.startswith('Record_'):
+            _recordTypes[globals()[name].TYPE] = globals()[name]
+
+    # Clear the iteration variable out of the class namespace so it
+    # doesn't become an attribute.
+    del name
+
+
     def lookupRecordType(self, type):
-        return globals().get('Record_' + QUERY_TYPES.get(type, ''), None)
+        """
+        Retrieve the L{IRecord} implementation for the given record type.
+
+        @param type: A record type, such as L{A} or L{NS}.
+        @type type: C{int}
+
+        @return: An object which implements L{IRecord} or C{None} if none
+            can be found for the given type.
+        @rtype: L{types.ClassType}
+        """
+        return self._recordTypes.get(type, None)
 
 
     def toStr(self):
@@ -1559,6 +1637,8 @@ class Message:
         strio = StringIO.StringIO(str)
         self.decode(strio)
 
+
+
 class DNSMixin(object):
     """
     DNS protocol mixin shared by UDP and TCP implementations.
@@ -1819,4 +1899,3 @@ class DNSProtocol(DNSMixin, protocol.Protocol):
         """
         id = self.pickID()
         return self._query(queries, timeout, id, self.writeMessage)
-
