@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -585,6 +585,14 @@ class ClientTestCase(unittest.TestCase):
         d.addCallback(self.checkResult, dns.TXT)
         return d
 
+    def test_lookupSenderPolicy(self):
+        """
+        See L{test_lookupAddress}
+        """
+        d = client.lookupSenderPolicy(self.hostname)
+        d.addCallback(self.checkResult, dns.SPF)
+        return d
+
     def test_lookupResponsibility(self):
         """
         See L{test_lookupAddress}
@@ -633,3 +641,23 @@ class ClientTestCase(unittest.TestCase):
         d = client.lookupNamingAuthorityPointer(self.hostname)
         d.addCallback(self.checkResult, dns.NAPTR)
         return d
+
+
+class ThreadedResolverTests(unittest.TestCase):
+    """
+    Tests for L{client.ThreadedResolver}.
+    """
+    def test_deprecated(self):
+        """
+        L{client.ThreadedResolver} is deprecated.  Instantiating it emits a
+        deprecation warning pointing at the code that does the instantiation.
+        """
+        client.ThreadedResolver()
+        warnings = self.flushWarnings(offendingFunctions=[self.test_deprecated])
+        self.assertEquals(
+            warnings[0]['message'],
+            "twisted.names.client.ThreadedResolver is deprecated since "
+            "Twisted 9.0, use twisted.internet.base.ThreadedResolver "
+            "instead.")
+        self.assertEquals(warnings[0]['category'], DeprecationWarning)
+        self.assertEquals(len(warnings), 1)
