@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.names.test.test_names -*-
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -73,6 +73,8 @@ test_domain_com = NoFileAuthority(
             soa_record,
             dns.Record_A('127.0.0.1'),
             dns.Record_NS('39.28.189.39'),
+            dns.Record_SPF('v=spf1 mx/30 mx:example.org/30 -all'),
+            dns.Record_SPF('v=spf1 +mx a:\0colo', '.example.com/28 -all not valid'),
             dns.Record_MX(10, 'host.test-domain.com'),
             dns.Record_HINFO(os='Linux', cpu='A Fast One, Dontcha know'),
             dns.Record_CNAME('canonical.name.com'),
@@ -221,13 +223,14 @@ class ServerDNSTestCase(unittest.TestCase):
         )
 
 
-    def testAdressRecord3(self):
+    def testAddressRecord3(self):
         """Test DNS 'A' record queries with edge cases"""
         return self.namesTest(
             self.resolver.lookupAddress('host-two.test-domain.com'),
             [dns.Record_A('255.255.255.254', ttl=19283784), dns.Record_A('0.0.0.0', ttl=19283784)]
         )
 
+
     def testAuthority(self):
         """Test DNS 'SOA' record queries"""
         return self.namesTest(
@@ -345,6 +348,17 @@ class ServerDNSTestCase(unittest.TestCase):
         )
 
 
+    def test_spf(self):
+        """
+        L{DNSServerFactory} can serve I{SPF} resource records.
+        """
+        return self.namesTest(
+            self.resolver.lookupSenderPolicy('test-domain.com'),
+            [dns.Record_SPF('v=spf1 mx/30 mx:example.org/30 -all', ttl=19283784),
+            dns.Record_SPF('v=spf1 +mx a:\0colo', '.example.com/28 -all not valid', ttl=19283784)]
+        )
+
+
     def testWKS(self):
         """Test DNS 'WKS' record queries"""
         return self.namesTest(
@@ -600,6 +614,23 @@ class HostsTestCase(unittest.TestCase):
                                   dns.DomainError)
 
 
+    def test_searchFileFor(self):
+        """
+        L{searchFileFor} parses hosts(5) files and returns the address for
+        the given name, or C{None} if the name is not found.
+        """
+        tmp = self.mktemp()
+        f = open(tmp, 'w')
+        f.write('127.0.1.1	helmut.example.org	helmut\n')
+        f.write('# a comment\n')
+        f.write('::1     localhost ip6-localhost ip6-loopback\n')
+        f.close()
+        self.assertEquals(hosts.searchFileFor(tmp, 'helmut'), '127.0.1.1')
+        self.assertEquals(hosts.searchFileFor(tmp, 'ip6-localhost'), '::1')
+        self.assertIdentical(hosts.searchFileFor(tmp, 'blah'), None)
+
+
+
 class FakeDNSDatagramProtocol(object):
     def __init__(self):
         self.queries = []
@@ -750,3 +781,117 @@ class FilterAnswersTests(unittest.TestCase):
         L{DNSUnknownError}.
         """
         return self._rcodeTest(EREFUSED + 1, DNSUnknownError)
+
+
+
+class AuthorityTests(unittest.TestCase):
+    """
+    Tests for the basic response record selection code in L{FileAuthority}
+    (independent of its fileness).
+    """
+    def test_recordMissing(self):
+        """
+        If a L{FileAuthority} has a zone which includes an I{NS} record for a
+        particular name and that authority is asked for another record for the
+        same name which does not exist, the I{NS} record is not included in the
+        authority section of the response.
+        """
+        authority = NoFileAuthority(
+            soa=(str(soa_record.mname), soa_record),
+            records={
+                str(soa_record.mname): [
+                    soa_record,
+                    dns.Record_NS('1.2.3.4'),
+                    ]})
+        d = authority.lookupAddress(str(soa_record.mname))
+        result = []
+        d.addCallback(result.append)
+        answer, authority, additional = result[0]
+        self.assertEquals(answer, [])
+        self.assertEquals(
+            authority, [
+                dns.RRHeader(
+                    str(soa_record.mname), soa_record.TYPE,
+                    ttl=soa_record.expire, payload=soa_record,
+                    auth=True)])
+        self.assertEquals(additional, [])
+
+
+    def _referralTest(self, method):
+        """
+        Create an authority and make a request against it.  Then verify that the
+        result is a referral, including no records in the answers or additional
+        sections, but with an I{NS} record in the authority section.
+        """
+        subdomain = 'example.' + str(soa_record.mname)
+        nameserver = dns.Record_NS('1.2.3.4')
+        authority = NoFileAuthority(
+            soa=(str(soa_record.mname), soa_record),
+            records={
+                subdomain: [
+                    nameserver,
+                    ]})
+        d = getattr(authority, method)(subdomain)
+        result = []
+        d.addCallback(result.append)
+        answer, authority, additional = result[0]
+        self.assertEquals(answer, [])
+        self.assertEquals(
+            authority, [dns.RRHeader(
+                    subdomain, dns.NS, ttl=soa_record.expire,
+                    payload=nameserver, auth=False)])
+        self.assertEquals(additional, [])
+
+
+    def test_referral(self):
+        """
+        When an I{NS} record is found for a child zone, it is included in the
+        authority section of the response.  It is marked as non-authoritative if
+        the authority is not also authoritative for the child zone (RFC 2181,
+        section 6.1).
+        """
+        self._referralTest('lookupAddress')
+
+
+    def test_allRecordsReferral(self):
+        """
+        A referral is also generated for a request of type C{ALL_RECORDS}.
+        """
+        self._referralTest('lookupAllRecords')
+
+
+
+class NoInitialResponseTestCase(unittest.TestCase):
+
+    def test_no_answer(self):
+        """
+        If a request returns a L{dns.NS} response, but we can't connect to the
+        given server, the request fails with the error returned at connection.
+        """
+
+        def query(self, *args):
+            # Pop from the message list, so that it blows up if more queries
+            # are run than expected.
+            return succeed(messages.pop(0))
+
+        def queryProtocol(self, *args, **kwargs):
+            return defer.fail(socket.gaierror("Couldn't connect"))
+
+        resolver = Resolver(servers=[('0.0.0.0', 0)])
+        resolver._query = query
+        messages = []
+        # Let's patch dns.DNSDatagramProtocol.query, as there is no easy way to
+        # customize it.
+        self.patch(dns.DNSDatagramProtocol, "query", queryProtocol)
+
+        records = [
+            dns.RRHeader(name='fooba.com', type=dns.NS, cls=dns.IN, ttl=700,
+                         auth=False,
+                         payload=dns.Record_NS(name='ns.twistedmatrix.com',
+                         ttl=700))]
+        m = dns.Message(id=999, answer=1, opCode=0, recDes=0, recAv=1, auth=1,
+                        rCode=0, trunc=0, maxSize=0)
+        m.answers = records
+        messages.append(m)
+        return self.assertFailure(
+            resolver.getHostByName("fooby.com"), socket.gaierror)
